@@ -1,14 +1,19 @@
 import functools
-import unittest
+import locale
+import io
 import os
 import shutil
 import subprocess
 import sys
 import tempfile
+import unittest
+from contextlib import redirect_stdout
 from pathlib import Path
 from unittest.mock import patch
 from shell_lib.shell_lib import FS_ENCODING, Shell, PathInfo
 
+IS_KNOWN_PYTHON_VERSION = (sys.version_info < (3, 15))
+
 def optional_exception(exception_type, expected_message):
     def decorator(func):
         @functools.wraps(func)
@@ -44,11 +49,11 @@ class TestShellFileOperations(unittest.TestCase):
         self.sub_file_path = self.dir_path / 'sub_file.txt'
 
         # Create some files and directories for testing
-        with open(self.file_path, 'w') as f:
-            f.write("Hello, World!")
+        with open(self.file_path, 'wb') as f:
+            f.write(b"Hello, World!")
         os.makedirs(self.dir_path, exist_ok=True)
-        with open(self.sub_file_path, 'w') as f:
-            f.write("Sub file content.")
+        with open(self.sub_file_path, 'wb') as f:
+            f.write(b"Sub file content.")
 
     def tearDown(self):
         shutil.rmtree(self.test_dir)
@@ -62,7 +67,7 @@ class TestShellFileOperations(unittest.TestCase):
             path = func(self.file_path)
             self.assertEqual(self.sh.path(path), self.file_path)
 
-        if sys.version_info[:2] <= (3, 13):
+        if IS_KNOWN_PYTHON_VERSION:
             # need to convert bytes path to str
             with self.assertRaises(TypeError):
                 Path(b'abc')
@@ -86,6 +91,16 @@ class TestShellFileOperations(unittest.TestCase):
             self.sh.remove_dir(self.file_path)
         self.assertTrue(os.path.isfile(self.file_path))
 
+    def test_create_dir_multi_level(self):
+        dir = self.test_dir / 'a' / 'b' / 'c'
+        self.sh.create_dir(dir)
+        self.assertTrue(os.path.isdir(dir))
+
+        with self.assertRaises(FileExistsError):
+            self.sh.create_dir(dir)
+
+        self.sh.create_dir(dir, exist_ok=True)
+
     def test_create_dir_exist_ok(self):
         # keyword only arg
         with self.assertRaises(TypeError):
@@ -194,12 +209,13 @@ class TestShellFileOperations(unittest.TestCase):
             self.assertTrue(os.path.isfile(sub_dir_file))
             self.assertFalse(os.path.islink(sub_dir_file))
 
-            # On Windows, need privilege.
-            # https://docs.python.org/3/library/os.html#os.symlink
+            # symlink in sub-dir
             dir_symlink = dir / 'symlink'
-            if not self.sh.is_os(self.sh.OS_Windows):
-                # symlink in sub-dir
+            try:
                 os.symlink(target_dir, dir_symlink, target_is_directory=True)
+            except OSError as e:
+                print("Can't create symlink in test_clear_dir", e)
+            else:
                 self.assertTrue(os.path.isdir(dir_symlink))
                 self.assertTrue(os.path.islink(dir_symlink))
                 self.assertFalse(os.path.isfile(dir_symlink))
@@ -207,6 +223,9 @@ class TestShellFileOperations(unittest.TestCase):
                 self.assertTrue(fi.is_dir)
                 self.assertTrue(fi.is_link)
                 self.assertFalse(fi.is_file)
+                self.assertTrue(self.sh.is_dir(dir_symlink))
+                self.assertTrue(self.sh.is_link(dir_symlink))
+                self.assertFalse(self.sh.is_file(dir_symlink))
 
             # clear
             self.sh.clear_dir(func(dir))
@@ -238,7 +257,24 @@ class TestShellFileOperations(unittest.TestCase):
         with self.assertRaises(FileNotFoundError):
             self.sh.rename_dir(src, dst)
 
-    def test_dst_behavior(self):
+    def test_dst_is_existing_file(self):
+        src = self.dir_path
+        dst = self.file_path
+
+        sh = self.sh
+        # all _dir methods raise FileExistsError
+        with self.assertRaises(FileExistsError):
+            sh.copy_dir(src, dst)
+        with self.assertRaises(FileExistsError):
+            sh.copy_dir(src, dst, remove_existing_dst=True)
+        with self.assertRaises(FileExistsError):
+            sh.move_dir(src, dst)
+        with self.assertRaises(FileExistsError):
+            sh.move_dir(src, dst, remove_existing_dst=True)
+        with self.assertRaises(FileExistsError):
+            sh.rename_dir(src, dst)
+
+    def test_dst_is_existing_dir(self):
         class Prepare:
             src_file = self.file_path
             src_dir  = self.test_dir / 'src_dir'
@@ -636,6 +672,22 @@ class TestShellFileOperations(unittest.TestCase):
             self.assertTrue(os.path.isfile(dst / 'sub_file.txt'))
             src = dst
 
+    def test_get_file_size(self):
+        sh = self.sh
+        # not exist
+        with self.assertRaises(FileNotFoundError):
+            sh.get_file_size(self.test_dir / 'not_exist')
+        # is dir
+        with self.assertRaises(IsADirectoryError):
+            sh.get_file_size(self.dir_path)
+
+        path = self.test_dir / 'len_bytes.txt'
+        LEN = 123
+        with open(path, 'wb') as f:
+            f.write(b'a' * LEN)
+        for func in (path_noop, str, path2bytes):
+            self.assertEqual(sh.get_file_size(func(path)), LEN)
+
     def test_get_path_info_for_file(self):
         # file
         info = self.sh.get_path_info(self.file_path)
@@ -645,10 +697,9 @@ class TestShellFileOperations(unittest.TestCase):
         self.assertTrue(info.is_file)
         self.assertFalse(info.is_dir)
         self.assertFalse(info.is_link)
-        if self.sh.is_os(self.sh.OS_Windows):
-            self.assertRegex(info.permissions, r'^[0-7]$')
-        else:
-            self.assertRegex(info.permissions, r'^[0-7]{3}$')
+        self.assertIs(info.is_readable, True)
+        self.assertIs(info.is_writable, True)
+        self.assertEqual(type(info.is_executable), bool)
 
         # dir
         info = self.sh.get_path_info(self.dir_path)
@@ -658,10 +709,9 @@ class TestShellFileOperations(unittest.TestCase):
         self.assertFalse(info.is_file)
         self.assertTrue(info.is_dir)
         self.assertFalse(info.is_link)
-        if self.sh.is_os(self.sh.OS_Windows):
-            self.assertRegex(info.permissions, r'^[0-7]$')
-        else:
-            self.assertRegex(info.permissions, r'^[0-7]{3}$')
+        self.assertIs(info.is_readable, True)
+        self.assertIs(info.is_writable, True)
+        self.assertEqual(type(info.is_executable), bool)
 
         # not existing
         with self.assertRaises(FileNotFoundError):
@@ -670,27 +720,45 @@ class TestShellFileOperations(unittest.TestCase):
         self.assertRegex(repr(info),
                          (r"PathInfo\(path=.*?, size=.*?, "
                           r"ctime=.*?, mtime=.*?, atime=.*?, "
-                          r"is_dir=.*?, is_file=.*?, permissions=.*?\)"))
+                          r"is_dir=.*?, is_file=.*?, is_link=.*?, "
+                          r"is_readable=.*?, is_writable=.*?, is_executable=.*?\)"))
 
         with self.assertRaises(FileNotFoundError):
             self.sh.get_path_info(self.test_dir / 'non_existent')
 
-        # On Windows, need privilege to create symlink.
-        # https://docs.python.org/3/library/os.html#os.symlink
-        if not self.sh.is_os(self.sh.OS_Windows):
-            # symlink for file
-            symlink_file = self.test_dir / 'symlink_file'
+        # symlink for file
+        symlink_file = self.test_dir / 'symlink_file'
+        try:
             os.symlink(self.file_path, symlink_file, target_is_directory=False)
+        except OSError as e:
+            print("Can't create file symlink in test_get_path_info_for_file", e)
+        else:
+            # os.path.is*
             self.assertTrue(os.path.isfile(symlink_file))
             self.assertTrue(os.path.islink(symlink_file))
             self.assertFalse(os.path.isdir(symlink_file))
+            # get_path_info
             pi = self.sh.get_path_info(symlink_file)
             self.assertTrue(pi.is_file)
             self.assertTrue(pi.is_link)
             self.assertFalse(pi.is_dir)
-            # symlink for dir
-            symlink_dir = self.test_dir / 'symlink_dir'
+            # sh.is_*
+            self.assertTrue(self.sh.is_file(symlink_file))
+            self.assertTrue(self.sh.is_link(symlink_file))
+            self.assertFalse(self.sh.is_dir(symlink_file))
+            # sh.get_file_size
+            with open(self.file_path, 'rb') as f:
+                size = len(f.read())
+            self.assertEqual(pi.size, size)
+            self.assertEqual(self.sh.get_file_size(symlink_file), size)
+
+        # symlink for dir
+        symlink_dir = self.test_dir / 'symlink_dir'
+        try:
             os.symlink(self.dir_path, symlink_dir, target_is_directory=True)
+        except OSError as e:
+            print("Can't create dir symlink in test_get_path_info_for_file", e)
+        else:
             self.assertTrue(os.path.isdir(symlink_dir))
             self.assertTrue(os.path.islink(symlink_dir))
             self.assertFalse(os.path.isfile(symlink_dir))
@@ -698,6 +766,9 @@ class TestShellFileOperations(unittest.TestCase):
             self.assertTrue(pi.is_dir)
             self.assertTrue(pi.is_link)
             self.assertFalse(pi.is_file)
+            self.assertTrue(self.sh.is_dir(symlink_dir))
+            self.assertTrue(self.sh.is_link(symlink_dir))
+            self.assertFalse(self.sh.is_file(symlink_dir))
 
     def test_get_path_info_types(self):
         for func in (path_noop, str, path2bytes):
@@ -789,6 +860,9 @@ class TestShellFileOperations(unittest.TestCase):
         self.assertFalse(self.sh.path_exists(self.test_dir / 'non_existent'))
 
     def test_is_file(self):
+        # not exist
+        self.assertIs(self.sh.is_file(self.test_dir / 'not_exist'), False)
+
         for func in (path_noop, str, path2bytes):
             self.assertTrue(self.sh.is_file(func(self.file_path)))
             self.assertTrue(self.sh.path_exists(func(self.file_path)))
@@ -796,12 +870,25 @@ class TestShellFileOperations(unittest.TestCase):
             self.assertTrue(self.sh.path_exists(func(self.dir_path)))
 
     def test_is_dir(self):
+        # not exist
+        self.assertIs(self.sh.is_dir(self.test_dir / 'not_exist'), False)
+
         for func in (path_noop, str, path2bytes):
             self.assertTrue(self.sh.is_dir(func(self.dir_path)))
             self.assertTrue(self.sh.path_exists(func(self.dir_path)))
             self.assertFalse(self.sh.is_dir(func(self.file_path)))
             self.assertTrue(self.sh.path_exists(func(self.file_path)))
 
+    def test_is_link(self):
+        # not exist
+        self.assertIs(self.sh.is_link(self.test_dir / 'not_exist'), False)
+
+        for func in (path_noop, str, path2bytes):
+            self.assertFalse(self.sh.is_link(func(self.dir_path)))
+            self.assertTrue(self.sh.path_exists(func(self.dir_path)))
+            self.assertFalse(self.sh.is_link(func(self.file_path)))
+            self.assertTrue(self.sh.path_exists(func(self.file_path)))
+
     def test_split_path(self):
         for func in (path_noop, str, path2bytes):
             parent_dir, filename = self.sh.split_path(func(self.file_path))
@@ -812,6 +899,9 @@ class TestShellFileOperations(unittest.TestCase):
                 self.assertEqual(parent_dir, path2bytes(self.test_dir))
                 self.assertEqual(filename, b'test_file.txt')
 
+        self.assertTrue(self.sh.split_path(self.file_path)[1], 'test_file.txt')
+        self.assertTrue(self.sh.split_path(self.dir_path)[1], 'test_dir')
+
     def test_join_path(self):
         for func in (path_noop, str, path2bytes):
             part1 = func(self.test_dir)
@@ -839,6 +929,8 @@ class TestShellOtherOperations(unittest.TestCase):
             self.sh.aaa = 123
         with self.assertRaises(AttributeError):
             self.sh.OS_Linux = 123
+        with self.assertRaises(AttributeError):
+            delattr(self.sh, 'cd')
 
     @patch('subprocess.run')
     def test_call_executes_shell_command(self, mock_run):
@@ -869,10 +961,9 @@ class TestShellOtherOperations(unittest.TestCase):
 
     # --- Script Control API Mocked Tests ---
     @patch('builtins.input', side_effect=['invalid', '4', '0', '2 '])
-    @patch('builtins.print')
-    def test_ask_choice(self, mock_print, mock_input):
+    def test_ask_choice(self, mock_input):
         choice = self.sh.ask_choice("Choose an option:",
-                                     "Option 1", "Option 2", "Option 3")
+                                    "Option 1", "Option 2", "Option 3")
         self.assertEqual(choice, 2)
         self.assertEqual(mock_input.call_count, 4)
 
@@ -884,8 +975,7 @@ class TestShellOtherOperations(unittest.TestCase):
                                           '', ' YeS ',
                                           '1', 'y ',
                                           'ys', ' Y'])
-    @patch('builtins.print')
-    def test_ask_yes_no_yes(self, mock_print, mock_input):
+    def test_ask_yes_no_yes(self, mock_input):
         self.assertTrue(self.sh.ask_yes_no("Do you agree?"))
         self.assertTrue(self.sh.ask_yes_no("Do you agree?"))
         self.assertTrue(self.sh.ask_yes_no("Do you agree?"))
@@ -896,8 +986,7 @@ class TestShellOtherOperations(unittest.TestCase):
                                           '', ' No ',
                                           '1', 'n ',
                                           'not', ' N'])
-    @patch('builtins.print')
-    def test_ask_yes_no_no(self, mock_print, mock_input):
+    def test_ask_yes_no_no(self, mock_input):
         self.assertFalse(self.sh.ask_yes_no("Do you agree?"))
         self.assertFalse(self.sh.ask_yes_no("Do you agree?"))
         self.assertFalse(self.sh.ask_yes_no("Do you agree?"))
@@ -906,8 +995,7 @@ class TestShellOtherOperations(unittest.TestCase):
 
     @patch('builtins.input', side_effect=['d.3.2', '', '1.3.2',
                                           'abc ', ' 1.3.2 '])
-    @patch('builtins.print')
-    def test_ask_regex_input(self, mock_print, mock_input):
+    def test_ask_regex_input(self, mock_input):
         m = self.sh.ask_regex_input("Input version", r"\s*(\d+\.\d+\.\d+)\s*")
         self.assertEqual(m.group(1), '1.3.2')
 
@@ -933,21 +1021,220 @@ class TestShellOtherOperations(unittest.TestCase):
         mock_exit.assert_called_once_with(91)
 
     # --- Utility and OS-specific Tests ---
-    @optional_exception(RuntimeError, 'Unable to get')
-    def test_get_username(self):
-        self.assertEqual(type(self.sh.get_username()), str)
+    # Pass on Simplified Chinese Windows 10 and Ubuntu 24.04.
+    @optional_exception(UnicodeError, '')
+    def test_get_set_env_non_ascii(self):
+        sh = self.sh
+        K = '环境变量shell-lib'
+        V = '环境变量值'
+        # non-ascii env name works for posix (inside process), windows.
+        orig = sh.get_env(K)
+        self.assertEqual(orig, os.getenv(K))
+        with sh.set_env(K, V):
+            self.assertEqual(sh.get_env(K), V)
+            self.assertEqual(os.getenv(K), V)
+        self.assertEqual(sh.get_env(K), orig)
+        self.assertEqual(os.getenv(K), orig)
 
-    @optional_exception(RuntimeError, 'Unable to get')
-    def test_is_elevated(self):
-        self.assertEqual(type(self.sh.is_elevated()), bool)
+        cmd = f"import os;print(os.getenv('{K}'),end='',flush=1)"
 
-    def test_get_preferred_encoding(self):
-        self.assertEqual(type(self.sh.get_preferred_encoding()), str)
+        # on posix, non-ascii env name works for: shell=False
+        with sh.set_env(K, V):
+            ret = sh.safe_run([sys.executable, '-c', cmd], print_output=False)
+            self.assertEqual(ret.stdout, V)
+        self.assertEqual(sh.get_env(K), orig)
+        self.assertEqual(os.getenv(K), orig)
 
-    def test_get_filesystem_encoding(self):
-        self.assertEqual(type(self.sh.get_filesystem_encoding()), str)
-        self.assertEqual(self.sh.get_filesystem_encoding(),
-                         sys.getfilesystemencoding())
+        # on posix, non-ascii env name doesn't work for: shell=True
+        # posix env name only accept (a-z, A-Z, 0-9, _)
+        # otherwise, child process can't see the env.
+        if sh.is_os(sh.OS_Windows):
+            with sh.set_env(K, V):
+                ret = sh(f'{sys.executable} -c "{cmd}"', print_output=False)
+                self.assertEqual(ret.stdout, V)
+            self.assertEqual(sh.get_env(K), orig)
+            self.assertEqual(os.getenv(K), orig)
+
+    def test_get_set_env(self):
+        sh = self.sh
+        K = 'SHELL_LIB_TEST_KEY'
+        V = 'SHELL_LIB_TEST_VALUE'
+        DEFAULT = 'default_value'
+        sh.set_env(K, V)
+        self.assertEqual(sh.get_env(K), V)
+        self.assertEqual(sh.get_env(K, DEFAULT), V)
+        self.assertEqual(os.getenv(K), V)
+        for _ in range(2):
+            sh.set_env(K, None)
+            self.assertEqual(sh.get_env(K), None)
+            self.assertEqual(sh.get_env(K, DEFAULT), DEFAULT)
+            self.assertEqual(os.getenv(K), None)
+
+        TEST_2_K = 'SHELL_LIB_TEST_2_K'
+        TEST_2_V = 'SHELL_LIB_TEST_2_V'
+        self.assertEqual(sh.get_env(TEST_2_K), os.getenv(TEST_2_K))
+        os.environ[TEST_2_K] = TEST_2_V
+        self.assertEqual(sh.get_env(TEST_2_K), TEST_2_V)
+
+        TEST_3_K = 'SHELL_LIB_TEST_3_K'
+        TEST_3_V = 'SHELL_LIB_TEST_3_V'
+        self.assertEqual(sh.get_env(TEST_3_K), os.getenv(TEST_3_K))
+        os.putenv(TEST_3_K, TEST_3_V)
+        self.assertEqual(sh.get_env(TEST_3_K), TEST_3_V)
+
+        SUB_K = "SHELL_LIB_SUB_key"
+        SUB_V = "SHELL-LIB-SUB-value"
+        sh.set_env(SUB_K, SUB_V)
+
+        # subprocess, shell=False
+        ret = sh.safe_run(
+                [sys.executable, '-c',
+                 f'import os;print(os.getenv("{SUB_K}"),end="",flush=1)'],
+                print_output=False)
+        self.assertEqual(ret.stdout, SUB_V)
+
+        # subprocess, shell=True
+        ret = sh(
+            f'''{sys.executable} -c "import os;print(os.getenv('{SUB_K}'),end='',flush=1)"''',
+            print_output=False)
+        self.assertEqual(ret.stdout, SUB_V)
+
+        # wrong type
+        with self.assertRaises(TypeError):
+            sh.get_env(b'bytes')
+        with self.assertRaises(TypeError):
+            sh.get_env(123)
+        with self.assertRaises(TypeError):
+            sh.get_env('A', 'B', 'C')
+
+        with self.assertRaises(TypeError):
+            sh.set_env(b'bytes', V)
+        with self.assertRaises(TypeError):
+            sh.set_env(K, b'bytes')
+        with self.assertRaises(TypeError):
+            sh.set_env(123, V)
+        with self.assertRaises(TypeError):
+            sh.set_env(K, 123)
+        with self.assertRaises(TypeError):
+            sh.set_env('A', 'B', 'C')
+        with self.assertRaises(TypeError):
+            sh.set_env(key='A', value='B')
+
+    def test_set_env_context_manager_k_v(self):
+        sh = self.sh
+        K = 'SHELL_LIB_TEST_KEY'
+        V1 = 'SHELL_LIB_TEST_VALUE_1'
+        V2 = 'SHELL_LIB_TEST_VALUE_2'
+        V3 = 'SHELL_LIB_TEST_VALUE_3'
+
+        # V1, V2, V3
+        orig = os.getenv(K)
+        with sh.set_env(K, V1):
+            self.assertEqual(sh.get_env(K), V1)
+            self.assertEqual(os.getenv(K), V1)
+            with sh.set_env(K, V2):
+                self.assertEqual(sh.get_env(K), V2)
+                self.assertEqual(os.getenv(K), V2)
+                with sh.set_env(K, V3):
+                    self.assertEqual(sh.get_env(K), V3)
+                    self.assertEqual(os.getenv(K), V3)
+                self.assertEqual(sh.get_env(K), V2)
+                self.assertEqual(os.getenv(K), V2)
+            self.assertEqual(sh.get_env(K), V1)
+            self.assertEqual(os.getenv(K), V1)
+        self.assertEqual(sh.get_env(K), orig)
+        self.assertEqual(os.getenv(K), orig)
+
+        # None, V2, None
+        orig = os.getenv(K)
+        with sh.set_env(K, None):
+            self.assertEqual(sh.get_env(K), None)
+            self.assertEqual(os.getenv(K), None)
+            with sh.set_env(K, V2):
+                self.assertEqual(sh.get_env(K), V2)
+                self.assertEqual(os.getenv(K), V2)
+                with sh.set_env(K, None):
+                    self.assertEqual(sh.get_env(K), None)
+                    self.assertEqual(os.getenv(K), None)
+                self.assertEqual(sh.get_env(K), V2)
+                self.assertEqual(os.getenv(K), V2)
+            self.assertEqual(sh.get_env(K), None)
+            self.assertEqual(os.getenv(K), None)
+        self.assertEqual(sh.get_env(K), orig)
+        self.assertEqual(os.getenv(K), orig)
+
+        # None, None, None
+        orig = os.getenv(K)
+        with sh.set_env(K, None):
+            self.assertEqual(sh.get_env(K), None)
+            self.assertEqual(os.getenv(K), None)
+            with sh.set_env(K, None):
+                self.assertEqual(sh.get_env(K), None)
+                self.assertEqual(os.getenv(K), None)
+                with sh.set_env(K, None):
+                    self.assertEqual(sh.get_env(K), None)
+                    self.assertEqual(os.getenv(K), None)
+                self.assertEqual(sh.get_env(K), None)
+                self.assertEqual(os.getenv(K), None)
+            self.assertEqual(sh.get_env(K), None)
+            self.assertEqual(os.getenv(K), None)
+        self.assertEqual(sh.get_env(K), orig)
+        self.assertEqual(os.getenv(K), orig)
+
+    def test_set_env_context_manager_dict(self):
+        sh = self.sh
+        K1 = 'SHELL_LIB_TEST_dict_k_1'
+        V1 = 'SHELL_LIB_TEST_dict_v_1'
+        K2 = 'SHELL_LIB_TEST_dict_k_2'
+        V2 = 'SHELL_LIB_TEST_dict_v_2'
+        K3 = 'SHELL_LIB_TEST_dict_k_3'
+        V3 = None
+
+        sh.set_env(K2, "1234")
+        sh.set_env(K3, "abcd")
+        orig_1 = sh.get_env(K1)
+        orig_2 = sh.get_env(K2)
+        orig_3 = sh.get_env(K3)
+        with sh.set_env({K1: V1, K2: V2, K3: V3}):
+            self.assertEqual(sh.get_env(K1), V1)
+            self.assertEqual(os.getenv(K1), V1)
+
+            self.assertEqual(sh.get_env(K2), V2)
+            self.assertEqual(os.getenv(K2), V2)
+
+            self.assertEqual(sh.get_env(K3), V3)
+            self.assertEqual(os.getenv(K3), V3)
+        self.assertEqual(sh.get_env(K1), orig_1)
+        self.assertEqual(sh.get_env(K2), orig_2)
+        self.assertEqual(sh.get_env(K3), orig_3)
+
+    def test_setenv_not_use_os_environ(self):
+        sh = self.sh
+        K = 'SHELL_LIB_TEST_NUOE_k'
+        V = 'SHELL_LIB_TEST_NUOE_v'
+        orig = sh.get_env(K)
+        self.assertEqual(orig, os.getenv(K))
+
+        # not use os.environ to set
+        os.putenv(K, V)
+        self.assertEqual(sh.get_env(K), V)
+        if IS_KNOWN_PYTHON_VERSION:
+            self.assertEqual(os.getenv(K), orig)
+
+        # unset
+        sh.set_env(K, None)
+        self.assertEqual(sh.get_env(K), orig)
+        self.assertEqual(os.getenv(K), orig)
+
+    def test_redirected_stdout(self):
+        K = 'SHELL_LIB_TEST_KEY'
+        V = 'SHELL_LIB_TEST_VALUE'
+
+        f = io.StringIO()
+        with redirect_stdout(f):
+            self.sh.set_env(K, V)
+        self.assertIn(f"Set environment variable '{K}' to '{V}'.",
+                      f.getvalue())
 
     def test_os_constants(self):
         a = ['OS_Windows', 'OS_Cygwin',
@@ -967,7 +1254,19 @@ class TestShellOtherOperations(unittest.TestCase):
                          self.sh.OS_Cygwin | self.sh.OS_Linux |
                          self.sh.OS_macOS | self.sh.OS_Unix)
 
-    def test_is_os_mock(self):
+        with self.assertRaises(AttributeError):
+            self.sh.OS_Windows = 123
+        with self.assertRaises(AttributeError):
+            del self.sh.OS_Linux
+
+    def test_z1_is_os(self):
+        os = ['OS_Windows', 'OS_Cygwin',
+              'OS_Linux', 'OS_macOS', 'OS_Unix', 'OS_Unix_like']
+        os = [i for i in os if self.sh.is_os(getattr(self.sh, i))]
+        print('sh.is_os():', os)
+
+        # mock tests
+        Shell._CURRENT_OS = None
         with patch('sys.platform', 'win32'):
             self.assertTrue(self.sh.is_os(self.sh.OS_Windows))
             self.assertFalse(self.sh.is_os(self.sh.OS_Cygwin))
@@ -999,5 +1298,36 @@ class TestShellOtherOperations(unittest.TestCase):
                 self.assertTrue(self.sh.is_os(self.sh.OS_Unix_like))
                 self.assertFalse(self.sh.is_os(self.sh.OS_Linux))
 
+    def test_z2_get_preferred_encoding(self):
+        enc = self.sh.get_preferred_encoding()
+        self.assertEqual(type(enc), str)
+        print('sh.get_preferred_encoding():', enc)
+
+    def test_z3_get_locale_encoding(self):
+        enc = self.sh.get_locale_encoding()
+        self.assertEqual(type(enc), str)
+        if sys.version_info >= (3, 11, 0, 'beta', 1):
+            self.assertEqual(enc, locale.getencoding())
+        print('sh.get_locale_encoding():', enc)
+
+    def test_z4_get_filesystem_encoding(self):
+        enc = self.sh.get_filesystem_encoding()
+        self.assertEqual(type(enc), str)
+        self.assertEqual(enc, sys.getfilesystemencoding())
+        print('sh.get_filesystem_encoding():', enc)
+
+    @optional_exception(RuntimeError, 'Unable to get')
+    def test_z5_get_username(self):
+        username = self.sh.get_username()
+        self.assertEqual(type(username), str)
+        print('sh.get_username():', username)
+        print('sh.home_dir():', self.sh.home_dir())
+
+    @optional_exception(RuntimeError, 'Unable to get')
+    def test_z6_is_elevated(self):
+        ret = self.sh.is_elevated()
+        self.assertEqual(type(ret), bool)
+        print('sh.is_elevated():', ret)
+
 if __name__ == '__main__':
     unittest.main()
\ No newline at end of file
